import logging

from fastapi import FastAPI
from tortoise import Tortoise, run_async
from tortoise.contrib.fastapi import register_tortoise

from src.app.config import get_settings

log = logging.getLogger("uvicorn")

models = [
    "src.epona.auth.models",
    "src.epona.pessoas.models",
]

settings = get_settings()

TORTOISE_ORM = {
    "connections": {"default": settings.database_url},
    "apps": {
        "models": {
            "models": models,
            "default_connection": "default",
        },
    },
}


def init_db(app: FastAPI) -> None:
    register_tortoise(
        app,
        db_url=settings.database_url,
        modules={"models": models},
        generate_schemas=True,
        add_exception_handlers=True,
    )


async def generate_schema() -> None:
    log.info("Initializing Tortoise...")

    await Tortoise.init(
        db_url=settings.database_url,
        modules={"models": models},
    )
    log.info("Generating database schema via Tortoise...")
    await Tortoise.generate_schemas()
    await Tortoise.close_connections()


if __name__ == "__main__":
    run_async(generate_schema())



import logging
import os
from datetime import datetime, timedelta
from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText
from functools import lru_cache
from typing import List, Optional, Union

import boto3
from botocore.exceptions import ClientError
from fastapi import Depends, HTTPException, status
from fastapi.security import OAuth2PasswordBearer, SecurityScopes
from jinja2 import Environment, PackageLoader
from jose import JWTError, jwt
from passlib.context import CryptContext
from tortoise.exceptions import IntegrityError

from . import models, schemas

admin_client = os.getenv("ADMIN_CLIENT", "admin")
algorithm = "HS256"
aws_region = os.getenv("AWS_DEFAULT_REGION")
env = Environment(loader=PackageLoader("epona", "auth"))
host = os.getenv("HOST_URL")
log = logging.getLogger("uvicorn")
oauth2_scheme = OAuth2PasswordBearer(tokenUrl="/auth/login")
pwd_context = CryptContext(schemes=["bcrypt"], deprecated="auto")
secret_key = os.getenv(
    "SECRET_KEY", "7a086308aff6f391b190ece729f81fdbe564efa21153cc99ce659be0ddcea596"
)


def valid_client(client: str) -> bool:
    """Verifica se é um client_id ou username válido"""
    if len(client) < 3 or len(client) > 20:
        return False
    if client.count(" ") > 0:
        return False
    return True


def valid_email(email: str) -> bool:
    """Verifica se a string é um email válido"""
    chars = ["@", ".", " "]
    at = 0
    dot = 0
    space = 0
    for c in chars:
        if c == "@":
            at = email.count(c)
        elif c == ".":
            dot = email.count(c)
        elif c == " ":
            space = email.count(c)
    if at == 1 and dot >= 1 and space == 0:
        return True
    return False


def valid_password(password: str) -> bool:
    """Verifica se a string é uma senha válida"""
    if len(password.strip()) < 6:
        return False
    if len(password.strip()) > 100:
        return False
    return True


def verify_password(plain_password: str, hashed_password: str) -> bool:
    """verifica criptografia da senha"""
    return pwd_context.verify(plain_password, hashed_password)


def get_password_hash(password: str) -> str:
    """criptografa senha"""
    return pwd_context.hash(password)


async def get_user(client_id: str, username: str) -> Union[schemas.UserSchema, None]:
    """recebe um client_id e username e retorna o usuario"""
    user = (
        await models.User.filter(client_id=client_id, username=username)
        .first()
        .values()
    )
    if user:
        return schemas.UserSchema(**user)
    return None


async def get_user_with_pass(
    username: str, client: str
) -> Union[schemas.UserSchema, None]:
    """Autentica o usuario e retorna usuario e suas permissões"""
    user = (
        await models.User.filter(username=username, client_id=client)
        .prefetch_related("permissions")
        .first()
    )

    if not user:
        return None

    permissions = await user.permissions.all()
    permissions_list = []
    for perm in permissions:
        await perm.fetch_related("permission")
        permissions_list.append(perm.permission.name)
    user.scope = user.scope + " " + " ".join(permissions_list)
    return schemas.UserSchema(**dict(user))


async def authenticate_user(
    username: str, password: str, client_id: str
) -> Union[schemas.UserSchema, None]:
    """Faz o login do usuairo"""
    user = await get_user_with_pass(username, client_id)
    if not user:
        return None
    if not verify_password(password, user.password_hash):
        return None
    return user


async def save(payload: schemas.UserSchema) -> Union[schemas.UserSchema, None]:
    """Salva um novo usuário e configura o client_id"""
    user = models.User(
        active=payload.active,
        client_id=payload.client_id,
        email=payload.email,
        username=payload.username,
        password_hash=payload.password_hash,
    )
    if (
        not valid_email(user.email)
        or not valid_password(user.password_hash)
        or not valid_client(user.client_id)
        or not valid_client(user.username)
    ):
        return None

    client = await models.Client.filter(name__iexact=payload.client_id).first()
    if payload.client_id in admin_client:
        user.scope = "admin super-admin"
    else:
        user.scope = "user pessoas:create"
    if not client:
        user.scope = "admin" if "admin" not in user.scope else user.scope
        await user.save()
        client = models.Client(
            name=payload.client_id,
            active=True,
            valid_domains=payload.email.split("@")[1],
            activity_sectors_id=payload.sector,
        )
        await client.save()
        permissions = await models.Permission.all()
        scopes = ""
        for permission in permissions:
            user_permission = models.UserPermission(
                client_id=user.client_id,
                entities="all",
                user=user,
                permission=permission,
            )
            await user_permission.save()
            scopes += f" {permission.name}"
        user.scope = user.scope + scopes
    elif (
        client.multiple_domains
        or client.valid_domains
        and user.email.split("@")[1] in client.valid_domains
    ):
        await user.save()
        # permite que o usuario possa criar pessoas do usuario
        permission = await models.Permission.filter(name="pessoas:create").first()
        user_permission = models.UserPermission(
            client_id=client.id, entities="all", user=user, permission=permission
        )
        await user_permission.save()
    else:
        return None
    return schemas.UserSchema(**dict(user))


def create_access_token(data: dict, expires_delta: Optional[timedelta] = None) -> str:
    """cria token de acesso"""
    to_encode = data.copy()
    if expires_delta:
        expire = datetime.utcnow() + expires_delta
    else:
        expire = datetime.utcnow() + timedelta(minutes=60 * 10)
    to_encode.update({"exp": expire})
    encoded_jwt = jwt.encode(to_encode, secret_key, algorithm=algorithm)
    return encoded_jwt


async def get_current_user(
    security_scopes: SecurityScopes, token: str = Depends(oauth2_scheme)
) -> schemas.UserSchema:
    """Verifica permissões de segurança do usuário"""
    if security_scopes.scopes:
        authenticate_value = f'Bearer scope="{security_scopes.scope_str}"'
    else:
        authenticate_value = "Bearer"
    credentials_exception = HTTPException(
        status_code=status.HTTP_401_UNAUTHORIZED,
        detail="Usuário ou senha inválidos",
        headers={"WWW-Authenticate": authenticate_value},
    )
    try:
        payload = jwt.decode(token, secret_key, algorithms=[algorithm])
        username: str = payload.get("sub")
        if username is None:
            raise credentials_exception
        token_scopes = payload.get("scopes", "")
        token_data = schemas.TokenData(username=username, scopes=token_scopes.split())
    except JWTError as err:
        log.error(f"JWT error: {err}")
        raise credentials_exception
    client_id, username = token_data.username.split(":")
    user = await get_user(client_id=client_id, username=username)
    if user is None:
        raise credentials_exception
    # verifica permissoes quando o usuario nao é admin
    if "admin" not in token_scopes:
        for scope in security_scopes.scopes:
            if scope not in token_data.scopes:
                raise HTTPException(
                    status_code=status.HTTP_401_UNAUTHORIZED,
                    detail="Acesso não autorizado",
                    headers={"WWW-Authenticate": authenticate_value},
                )
    user.scope = token_scopes
    return user


async def get_current_active_user(
    current_user: schemas.UserSchema = Depends(get_current_user),
) -> schemas.UserSchema:
    """Verifica se o usuário está listado como ativo"""
    if not current_user.active:
        raise HTTPException(status_code=400, detail="Usuário desativado")
    return current_user


async def entity_access(
    perm_name: str, entity_id: int, user: schemas.UserSchema
) -> bool:
    """
    Verifica se um usuário tem permissão para manipular uma entidade específica
    O esquema de uma permissão é o nome da permissão seguido da ação. exemplo:
    perm_name:create
    perm_name é o nome da entidade
    create é a ação
    """
    user_wp = (
        await models.User.filter(client_id=user.client_id, id=user.id)
        .first()
        .prefetch_related("permissions")
    )
    if not user_wp:
        return False
    for permission in user_wp.permissions.related_objects:
        await permission.fetch_related("permission")
        if permission.permission.name == perm_name:
            if permission.entities == "all":
                return True
            for entity in permission.entities.split():
                if int(entity) == entity_id:
                    return True
    return False


@lru_cache(maxsize=8)
async def user_permissions(client_id, user_id) -> models.User:
    """Traz um usuario com suas permissões"""
    result = (
        await models.User.filter(client_id=client_id, id=user_id)
        .first()
        .prefech_related("permissions")
    )
    return result


async def user_add_permission(
    payload: schemas.UserPermissionPayloadSchema, admin: schemas.UserSchema
) -> int:
    """Salva novas permissões do usuario e retorna o número de permissões salvas"""
    saved = 0
    user = await models.User.filter(
        username=payload.username, client_id=admin.client_id
    ).first()
    for name in payload.permissions:
        permission = await models.Permission.filter(name=name).first()
        if not permission or not user:
            continue

        user_permission = await models.UserPermission.filter(
            client_id=admin.client_id, user_id=user.id, permission_id=permission.id
        ).first()
        if user_permission:
            user_permission.entities = " ".join(payload.entities)
        else:
            user_permission = models.UserPermission(
                client_id=admin.client_id,
                permission=permission,
                user=user,
                entities=" ".join(payload.entities),
            )
        try:
            await user_permission.save()
            saved += 1
        except IntegrityError:
            user_permission = await models.UserPermission.filter(
                client_id=payload.client_id,
                user_id=user.id,
                permission_id=permission.id,
            ).first()
            if not user_permission:
                continue
            user_permission.entities = " ".join(payload.entities)
            await user_permission.save()
            saved += 1
    return saved


async def add_permission(payload: schemas.PermissionSchema) -> bool:
    """Cria permissões do usuário"""
    permission = models.Permission(name=payload.name, description=payload.description)
    await permission.save()
    if not permission.id:
        raise HTTPException
    # adiciona a nova permissao para usuarios administradores
    admin_users = await models.User.filter(scope="admin").all()
    for user in admin_users:
        user_perm = schemas.UserPermissionPayloadSchema(
            client_id=user.client_id,
            entities=["all"],
            permissions=[permission.name],
            username=user.username,
        )
        user_schema = schemas.UserSchema(**dict(user))
        await user_add_permission(user_perm, user_schema)
    return True


async def get_users(admin: schemas.UserSchema) -> Optional[List[schemas.UserSchema]]:
    """Retorna lista de usuários salvos"""
    users = await models.User.filter(client_id=admin.client_id).all()
    if not users:
        return None
    return [schemas.UserSchema(**dict(user)) for user in users]


async def get_user_detail(
    username: str, admin: schemas.UserSchema
) -> Optional[schemas.UserPermissionResponseSchema]:
    """Mostra permissões de um usuário para o administrador"""
    user = await models.User.filter(
        username=username, client_id=admin.client_id
    ).first()
    if not user:
        return None
    await user.fetch_related("permissions")
    permissions_list = []
    for perm in user.permissions:
        await perm.fetch_related("permission")
        perm_dict = dict(perm)
        perm_dict["name"] = perm.permission.name
        perm_dict["description"] = perm.permission.description
        permissions_list.append(perm_dict)
    user_dict = dict(user)
    uwp = schemas.UserPermissionResponseSchema(
        **{**user_dict, "permissions": permissions_list}
    )
    return uwp


async def delete(username: str, admin: schemas.UserSchema) -> bool:
    """Deleta o usuário pelo administrador"""
    user = await models.User.filter(
        username=username, client_id=admin.client_id
    ).first()
    if not user:
        return False
    await user.delete()
    return True


async def update(
    payload: schemas.UserForm, admin: schemas.UserSchema
) -> schemas.UserForm:
    """Atualiza dados cadastrais do usuario"""
    user = await models.User.filter(
        username=payload.username, client_id=admin.client_id
    ).first()

    # TODO: refatorar trocando ternario por if normal
    user.active = (
        payload.active
        if (payload.active or not payload.active) and "admin" in admin.scope
        else user.active
    )
    user.scope = (
        payload.scope
        if payload.scope in ["user", "admin"] and "admin" in admin.scope
        else "user"
    )
    user.email = (
        payload.email
        if valid_email(payload.email) and "admin" in admin.scope
        else user.email
    )
    if valid_password(payload.password) and (
        user.username == admin.username or "admin" in admin.scope
    ):
        user.password_hash = get_password_hash(payload.password)

    await user.save()
    return schemas.UserForm(**{**dict(user), "password": ""})


async def exclude_entity_permission(entity_name: str, pk: int):
    """Exclui uma entidade da lista de permissões de todos os usuários"""
    permissions = await models.Permission.filter(name__startswith=entity_name).all()
    for perm in permissions:
        users_permissions = await models.UserPermission.filter(
            permission_id=perm.id
        ).all()
        for user_p in users_permissions:
            entities = user_p.entities.split()
            if entities.count(str(pk)) > 0:
                entities.remove(str(pk))
                user_p.entities = " ".join(entities)
                await user_p.save()


async def include_entity_permission(
    entity_name: str, pk: int, user: schemas.UserSchema
):
    """Inclui uma entidade na lista de entidades que o usuário pode editar"""
    permissions = await models.Permission.filter(name__startswith=entity_name).all()
    for perm in permissions:
        users_permissions = await models.UserPermission.filter(
            permission_id=perm.id, user_id=user.id
        ).all()
        for user_p in users_permissions:
            if user_p.entities != "all":
                user_p.entities = user_p.entities + f" {pk}"
                await user_p.save()


async def get_permissions() -> List[schemas.PermissionSchema]:
    """Lista todas as permissões salvas"""
    permissions = await models.Permission.filter().all()
    return [schemas.PermissionSchema(**dict(perm)) for perm in permissions]


async def delete_user_permission(payload: schemas.UserPermissionPayloadSchema) -> int:
    """Deleta permissões de um usuário por um administrador"""
    user = await models.User.filter(username=payload.username).first()
    deleted = 0
    for permission in payload.permissions:
        perm = await models.Permission.filter(name=permission).first()
        user_perm = await models.UserPermission.filter(
            user_id=user.id, permission_id=perm.id
        ).first()
        if user_perm:
            await user_perm.delete()
            deleted += 1
    return deleted


def send_email(payload: schemas.EmailPayload):
    """Envia email para usuario"""
    client = boto3.client("ses", region_name=aws_region)

    # Cria o cabecalho e-mail
    msg = MIMEMultipart("mixed")
    msg["Subject"] = payload.subject
    msg["From"] = payload.sender
    msg["To"] = payload.recipient

    # Cria o conteudo multipart filho
    msg_body = MIMEMultipart("alternative")

    # Codifica o texto e o HTML para o padrao definido em CHARSET
    text_part = MIMEText(
        payload.body_text.encode(payload.charset), "plain", payload.charset
    )
    html_part = MIMEText(
        payload.html_text.encode(payload.charset), "html", payload.charset
    )

    msg_body.attach(text_part)
    msg_body.attach(html_part)

    # inclui o body na mensagem
    msg.attach(msg_body)

    try:
        response = client.send_raw_email(
            Source=payload.sender,
            Destinations=[payload.recipient],
            RawMessage={
                "Data": msg.as_string(),
            },
        )
    except ClientError as err:
        print(err.response["Error"]["Message"])
    else:
        return response


async def password_recover(
    payload: schemas.PasswordRecoverPayload,
    template_path: str
) -> schemas.EmailResponse:
    """Formato do email de recuperação de senha"""
    user = await models.User.filter(
        email=payload.email, client_id=payload.client_id
    ).first()
    if user is None:
        raise HTTPException(
            404,
            f"Não foi encontrado nenhum usuário com o e-mail '{payload.email}' "
            f"na empresa '{payload.client_id}'",
        )
    expires = timedelta(minutes=15)
    recover_token = create_access_token({"email": user.email}, expires)
    template = env.get_template(template_path)
    context = {
        "token": recover_token,
        "host": host,
        "username": user.username,
    }
    # noqa: W291
    param = schemas.EmailPayload(
        **{
            "sender": "Sigeli Epona <sigeli@eponaconsultoria.com.br>",
            "recipient": user.email,
            "subject": "Recuperação de senha",
            "body_text": f"Olá {user.username},\n\nPara recuperar sua senha clique "
            f"aqui: <{host}/recuperar-senha?{recover_token}>\n"
            f"Em caso de problemas com o link, acessa a pagina "
            f"<{host}/recuperar-senha>"
            f"e utilize este token '{recover_token}' para fazer a alteração de "
            f"sua senha. As aspas não fazem parte do token.\n\nSigeli",
            "html_text": template.render(context),
            "charset": "utf-8",
        }
    )
    resp = send_email(param)
    if not resp:
        raise HTTPException(404, "Falha no envio do e-mail.")
    return schemas.EmailResponse(
        **{
            "msg": "E-mail de recuperação enviado com sucesso",
            "status": "OK",
        }
    )


async def recovering(form_data: schemas.ChangePassword) -> schemas.EmailResponse:
    """Recupera senha de acesso do usuário"""
    try:
        token = jwt.decode(form_data.token, secret_key, algorithms=[algorithm])
    except JWTError:
        raise HTTPException(401, "Token inválido")
    if not valid_password(form_data.password):
        raise HTTPException(401, "Senha inválida. Coloque uma senha mais forte")
    if form_data.password != form_data.password_confirm:
        raise HTTPException(401, "Senhas não conferem")

    user = await models.User.filter(
        username=form_data.username, client_id=form_data.client_id, email=token["email"]
    ).first()
    if not user:
        raise HTTPException(401, "Usuário não encontrado")
    user.password_hash = get_password_hash(form_data.password)
    try:
        await user.save()
    except Exception as ex:
        print(ex)
    return schemas.EmailResponse(
        **{"msg": "Senha alterada com sucesso", "status": "OK"}
    )


async def change_password(
    form_data: schemas.ChangePassword, user: schemas.UserSchema
) -> schemas.EmailResponse:
    """Troca senha do usuário"""
    if not valid_password(form_data.password):
        raise HTTPException(401, "Senha inválida. Coloque uma senha mais forte")
    if form_data.password != form_data.password_confirm:
        raise HTTPException(401, "Senhas não conferem")
    user_wp = await models.User.filter(id=user.id).first()
    if user_wp is None:
        raise (401, "Usuário não encontrado")
    user_wp.password_hash = get_password_hash(form_data.password)
    try:
        await user_wp.save()
    except Exception as ex:
        print(ex)
    return schemas.EmailResponse(
        **{"msg": "Senha alterada com sucesso", "status": "OK"}
    )



from typing import List, Optional

from fastapi import Form
from pydantic import BaseModel

from .models import User


def hash_user(self: User):
    return hash(f"{self.client_id}{self.username}")


class UserSchema(BaseModel):
    id: int
    active: bool
    client_id: str
    email: str
    scope: str = ""
    sector: int = 1
    username: str
    password_hash: str
    permissions: List = []


UserSchema.__hash__ = hash_user


class Token(BaseModel):
    access_token: str
    token_type: str


class TokenData(BaseModel):
    username: Optional[str] = None
    scopes: List[str] = []


class UserForm(BaseModel):
    active: bool = True
    client_id: Optional[str] = None
    email: str
    scope: Optional[str] = ""
    sector: Optional[int] = 1
    password: str
    username: str

    @classmethod
    def as_form(
        cls,
        active: bool = Form(...),
        client_id: str = Form(...),
        email: str = Form(...),
        scope: Optional[str] = Form(...),
        sector: Optional[int] = Form(1),
        password: str = Form(...),
        username: str = Form(...),
    ):
        return cls(
            active=active,
            client_id=client_id,
            email=email,
            scope=scope,
            sector=sector,
            password=password,
            username=username,
        )


class PermissionSchema(BaseModel):
    id: Optional[int] = 0
    name: str
    description: str


class UserPermissionPayloadSchema(BaseModel):
    client_id: str
    entities: List[str]
    permissions: List[str]
    username: str


class PermissionResponseSchema(UserPermissionPayloadSchema):
    id: int


class UserPermissionSchema(BaseModel):
    client_id: str
    description: str
    entities: str
    name: str
    permission_id: int
    user_id: int


class UserPermissionResponseSchema(BaseModel):
    id: int
    client_id: str
    permissions: List[UserPermissionSchema]
    username: str


class ChangePassword(BaseModel):
    client_id: str
    password: str
    password_confirm: str
    username: str
    token: Optional[str]

    @classmethod
    def as_form(
        cls,
        client_id: str = Form(...),
        password: str = Form(...),
        password_confirm: str = Form(...),
        username: str = Form(...),
        token: str = Form(...),
    ) -> Form:
        return cls(
            client_id=client_id,
            password=password,
            password_confirm=password_confirm,
            username=username,
            token=token,
        )


class PasswordRecoverPayload(BaseModel):
    email: str
    client_id: str


class EmailPayload(BaseModel):
    sender: str
    recipient: str
    subject: str
    body_text: str
    html_text: Optional[str]
    charset: str


class EmailResponse(BaseModel):
    msg: str
    status: str



from enum import Enum
from typing import List, Optional

from pydantic import BaseModel
from tortoise.contrib.pydantic import pydantic_model_creator

from .models import Pessoa

PessoaSchema = pydantic_model_creator(Pessoa)


class TipoEnum(str, Enum):
    FISICA = "FISICA"
    JURIDICA = "JURIDICA"


class TipoDocumentoEnum(str, Enum):
    REGISTRO_GERAL = "REGISTRO_GERAL"
    INSCRICAO_ESTADUAL = "INSCRICAO_ESTADUAL"


class PessoaPayloadSchema(BaseModel):
    pai: Optional[int]
    cpf_cnpj: str
    documento: str = None
    email: Optional[str]
    nome: str
    nome_fantasia: str = None
    licenciavel: Optional[bool] = False
    principal: Optional[bool] = False
    tipo: TipoEnum
    tipo_documento: TipoDocumentoEnum = None


class PessoaResponseSchema(PessoaPayloadSchema):
    id: int


class TipoTelefoneEnum(str, Enum):
    RESIDENCIAL = "RESIDENCIAL"
    COMERCIAL = "COMERCIAL"
    CELULAR_PESSOAL = "CELULAR_PESSOAL"
    CELULAR_TRABALHO = "CELULAR_TRABALHO"


class InfoContatoPayloadSchema(BaseModel):
    email: Optional[str]
    pessoa_id: int
    telefone: Optional[str]
    tipo: Optional[TipoTelefoneEnum]


class InfoContatoResponseSchema(InfoContatoPayloadSchema):
    id: int


class PessoaMatrizSchema(PessoaResponseSchema):
    responsavel: str
    contatos: Optional[List[InfoContatoResponseSchema]]



from fastapi import APIRouter, Depends

from src.app.config import Settings, get_settings

router = APIRouter()


@router.get("/health-check")
async def health_check(settings: Settings = Depends(get_settings)):
    return {
        "status": "Running",
        "environment": settings.environment,
        "testing": settings.testing,
    }



import logging
import os
from functools import lru_cache

from pydantic import AnyUrl, BaseSettings

log = logging.getLogger("uvicorn")


_db_host = os.getenv("DB_HOST", "localhost")
_db_user = os.getenv("DB_USER", "postgres")
_db_name = os.getenv("DB_NAME", "core")
_db_password = os.getenv("DB_PASSWORD", "postgres")


class Settings(BaseSettings):
    aws_region: str = os.getenv("AWS_DEFAULT_REGION")
    database_url: AnyUrl = (
        f"postgres://{_db_user}:{_db_password}@{_db_host}:5432/{_db_name}"
    )
    environment: str = "dev"
    host: str = os.getenv("HOST_URL")
    limit: int = os.getenv("LIMIT", 200)
    testing: bool = 0


@lru_cache()
def get_settings() -> Settings:
    log.info("Loading configuration from environment")
    return Settings()



from tortoise import fields, models


class Pessoa(models.Model):
    id = fields.IntField(pk=True)
    client_id = fields.CharField(max_length=50, null=True)
    pessoa_id = fields.IntField(null=True)
    cpf_cnpj = fields.CharField(max_length=15)
    email = fields.CharField(max_length=100, null=True)
    nome = fields.CharField(max_length=100)
    tipo = fields.CharField(max_length=20)
    licenciavel = fields.BooleanField(default=False)
    principal = fields.BooleanField(default=False)
    tipo_documento = fields.CharField(max_length=20, null=True)
    documento = fields.CharField(max_length=20, null=True)
    nome_fantasia = fields.CharField(max_length=100, null=True)
    created_at = fields.DatetimeField(null=True)

    def __str__(self):
        return self.cpf_cnpj

    class Meta:
        table = "pessoas"


class InfoContato(models.Model):
    id = fields.IntField(pk=True)
    client_id = fields.CharField(max_length=50)
    email = fields.CharField(max_length=50, null=True)
    pessoa_id = fields.IntField()
    telefone = fields.CharField(max_length=50, null=True)
    tipo = fields.CharField(max_length=20, null=True)

    class Meta:
        table = "info_contatos"



import asyncio
import os
from typing import List, Union

from fastapi import HTTPException
from openpyxl import Workbook
from pydantic.error_wrappers import ValidationError

from src.epona.auth.schemas import UserSchema

from .models import InfoContato, Pessoa
from .schemas import (InfoContatoPayloadSchema, InfoContatoResponseSchema,
                      PessoaPayloadSchema, PessoaResponseSchema, PessoaSchema)

env = os.getenv("ENVIONMENT", "dev")


async def post(payload: PessoaPayloadSchema, user: UserSchema) -> int:
    """Cria e salva uma pessoa ou empresa no banco de dados e retorna seu id"""
    pessoa = Pessoa(
        client_id=user.client_id,
        pessoa_id=payload.pai,
        cpf_cnpj=payload.cpf_cnpj,
        email=payload.email,
        nome=payload.nome,
        tipo=payload.tipo,
        tipo_documento=payload.tipo_documento,
        documento=payload.documento,
        nome_fantasia=payload.nome_fantasia,
    )
    if env == "prod" and not validate_cpf_cnpj(pessoa):
        cpf_cnpj = "CPF" if pessoa.tipo == "FISICA" else "CNPJ"
        raise HTTPException(422, f"{cpf_cnpj} inválido")
    if payload.licenciavel and payload.tipo == "JURIDICA":
        pessoa.licenciavel = payload.licenciavel
    if payload.principal and payload.tipo == "JURIDICA":
        pessoa.principal = payload.principal
    try:
        await pessoa.save()
    except Exception as err:
        print(err)
    return pessoa.id


async def update(payload: PessoaPayloadSchema, user: UserSchema) -> Union[int, None]:
    """Altera as informações de uma pessoa ou empresa e retorna seu id"""
    pessoa = await Pessoa.filter(
        cpf_cnpj=payload.cpf_cnpj, client_id=user.client_id
    ).first()
    if not pessoa:
        return None
    pessoa.nome = payload.nome if payload.nome else pessoa.nome
    pessoa.cpf_cnpj = payload.cpf_cnpj if payload.cpf_cnpj else pessoa.cpf_cnpj
    pessoa.email = payload.email if payload.email else pessoa.email
    pessoa.documento = payload.documento if payload.documento else pessoa.documento
    pessoa.tipo_documento = (
        payload.tipo_documento if payload.tipo_documento else pessoa.tipo_documento
    )
    pessoa.nome_fantasia = (
        payload.nome_fantasia if payload.nome_fantasia else pessoa.nome_fantasia
    )
    if payload.tipo == "JURIDICA":
        if payload.licenciavel is True:
            pessoa.licenciavel = True
        if payload.principal is True:
            pessoa.principal = True

    await pessoa.save()
    return pessoa.id


async def save_xlsx(wb: Workbook, user: UserSchema) -> List[PessoaResponseSchema]:
    """Salva pessoas a partir de uma planilha xlsx"""
    await asyncio.sleep(0)
    pessoas = []
    try:
        for i, line in enumerate(wb["Pessoas"].iter_rows()):
            if i == 0 or not line[0].value:
                continue
            payload = PessoaPayloadSchema(
                **{
                    "cpf_cnpj": str(line[0].value).strip(),
                    "nome": line[1].value.strip(),
                    "tipo_documento": line[2].value,
                    "documento": line[3].value,
                    "tipo": line[4].value,
                    "email": line[5].value,
                    "nome_fantasia": line[6].value,
                    "licenciavel": line[7].value,
                    "principal": line[8].value,
                }
            )
            if env == "prod" and not validate_cpf_cnpj(payload):
                raise ValueError
            pessoas.append(payload)
    except (ValueError, KeyError, ValidationError):
        raise ValueError(
            f"Erro na linha {i+1}. Verifique se todas a colunas estão"
            f"preenchidas e com valores válidos, inclusive CPF/CNPJ"
        )
    except IndexError:
        raise ValueError(
            "Número de colunas na planilha é menor que o esperado. "
            "Utilize o arquivo modelo."
        )
    armazenadas = []
    for pessoa in pessoas:
        result = await post(pessoa, user)
        if result:
            armazenadas.append(PessoaResponseSchema(**{"id": result, **pessoa.dict()}))
    return armazenadas


async def get(pk: int, user: UserSchema) -> Union[PessoaSchema, None]:
    """retorna informações de uma pessoa ou empresa filtrada por id"""
    pessoa = await Pessoa.filter(id=pk, client_id=user.client_id).first()
    if not pessoa:
        return None
    return PessoaSchema(**dict(pessoa))


async def get_all(user: UserSchema, limit=100) -> List:
    """retorna todas as pessoas e empresas vinculádas a um client_id"""
    pessoas = (
        await Pessoa.filter(client_id=user.client_id)
        .all()
        .order_by("-created_at")
        .limit(limit)
        .values()
    )
    pessoas_list = [pessoa for pessoa in pessoas]
    return pessoas_list


async def delete(pk: int, user: UserSchema) -> bool:
    """apaga uma pessoa ou empresa do banco por id"""
    pessoa = await Pessoa.filter(id=pk, client_id=user.client_id).first()
    if pessoa:
        await pessoa.delete()
        return True
    return False


async def by_name(nome: str, user: UserSchema) -> List:
    """Filtra pessoas por nome"""
    pessoas = await Pessoa.filter(nome__icontains=nome, client_id=user.client_id).all()
    pessoas_list = [pessoa for pessoa in pessoas]

    return pessoas_list


async def by_cpf_cnpj(cpf_cnpj: str, user: UserSchema) -> Union[Pessoa, None]:
    """Filtra pessoas ou empresas pelo cpf ou cnpj"""
    pessoa = await Pessoa.filter(cpf_cnpj=cpf_cnpj, client_id=user.client_id).first()
    if not pessoa:
        return None
    return pessoa


async def find_principal(
    empresa_id: int, user: UserSchema
) -> Union[PessoaResponseSchema, None]:
    """Filtra empresas e retorna sua principal"""
    empresa = await Pessoa.filter(id=empresa_id, client_id=user.client_id).first()
    if not empresa:
        return None
    if empresa.principal:
        return PessoaResponseSchema(**dict(empresa))
    return await find_principal(empresa.pessoa_id, user)


async def save_contato(
    pk: int, payload: InfoContatoPayloadSchema, user: UserSchema
) -> Union[InfoContatoResponseSchema, None]:
    """Salva e atualiza informações de contato de uma pessoa"""
    if pk > 0:
        info_contato = await InfoContato.filter(id=pk, client_id=user.client_id).first()
        if not info_contato:
            return None
        info_contato.email = payload.email
        info_contato.telefone = payload.telefone
        info_contato.tipo = payload.tipo
    else:
        info_contato = InfoContato(
            client_id=user.client_id,
            email=payload.email,
            pessoa_id=payload.pessoa_id,
            telefone=payload.telefone,
            tipo=payload.tipo,
        )
    await info_contato.save()
    if not info_contato.id > 0:
        return None
    return InfoContatoResponseSchema(**dict(info_contato))


async def get_contato(
    pk: int, user: UserSchema
) -> Union[InfoContatoResponseSchema, None]:
    """Retorna as informççoes de contato de uma pessoa por id"""
    info_contato = await InfoContato.filter(id=pk, client_id=user.client_id).first()
    if not info_contato:
        return None
    return InfoContatoResponseSchema(**dict(info_contato))


async def get_contato_all(
    pessoa_id: int, user: UserSchema
) -> Union[List[InfoContatoResponseSchema], None]:
    """Retorna todas os contatos vinculados a uma pessoa"""
    infos_contato = await InfoContato.filter(
        pessoa_id=pessoa_id, client_id=user.client_id
    ).all()
    if not infos_contato:
        return None
    return [InfoContatoResponseSchema(**dict(info)) for info in infos_contato]


async def delete_contato(pk: int, user: UserSchema) -> bool:
    """Apaga as informações de contato de uma pessoa por id"""
    info_contato = await InfoContato.filter(id=pk, client_id=user.client_id).first()
    if not info_contato:
        return False
    result = await info_contato.delete()
    if result:
        return False
    return True


def validate_cpf_cnpj(pessoa: Pessoa) -> bool:
    """Filtra cpf ou cnpf e valida o número"""
    if pessoa.tipo == "FISICA":
        return validate_cpf(pessoa.cpf_cnpj)
    return validate_cnpj(pessoa.cpf_cnpj)


def validate_cpf(cpf: str) -> bool:
    """Verifica se a string é um cpf válido"""
    try:
        if len(cpf) != 11:
            return False
        base = cpf[:9]
        dv = cpf[9:]
        same_digit = True
        for i, digit in enumerate(base):
            if i < len(base) - 1 and digit != base[i + 1]:
                same_digit = False
                break
        if same_digit:
            return False
        dv1_sum = 0
        for i in range(len(base)):
            dv1_sum += int(base[i]) * (10 - i)
        dv1_mod = dv1_sum % 11
        dv1 = str(11 - dv1_mod) if 11 - dv1_mod < 10 else "0"
        base2 = base + dv1
        dv2_sum = 0
        for i in range(len(base2)):
            dv2_sum += int(base2[i]) * (11 - i)
        dv2_mod = dv2_sum % 11
        dv2 = str(11 - dv2_mod) if 11 - dv2_mod < 10 else "0"
        return dv[0] == dv1 and dv[1] == dv2
    except (IndexError, ValueError):
        return False


def validate_cnpj(cnpj: str) -> bool:
    """Verifica se a string é um cnpj válido"""
    try:
        if len(cnpj) != 14:
            return False
        base = cnpj[:12]
        dv = cnpj[12:]
        # cada digito do cnpj tem um peso
        weights = [5, 4, 3, 2, 9, 8, 7, 6, 5, 4, 3, 2]
        dv1_sum = 0
        for i in range(len(weights)):
            dv1_sum += int(base[i]) * weights[i]
        dv1_mod = dv1_sum % 11
        dv1 = str(11 - dv1_mod) if 11 - dv1_mod < 10 else "0"
        # como mais um digito vai ser validado, eh incluido mais um peso a lista
        weights.insert(0, 6)  #
        base2 = base + dv1
        dv2_sum = 0
        for i in range(len(weights)):
            dv2_sum += int(base2[i]) * weights[i]
        dv2_mod = dv2_sum % 11
        dv2 = str(11 - dv2_mod) if 11 - dv2_mod < 10 else "0"
        return dv[0] == dv1 and dv[1] == dv2
    except (IndexError, ValueError):
        return False



import logging

from fastapi import FastAPI

from src.epona.auth import routers as auth
from src.epona.pessoas import routers as pessoas

from .routes import ping

log = logging.getLogger("uvicorn")


def create_application() -> FastAPI:
    application = FastAPI(name="api-core")

    application.include_router(auth.router, prefix="/auth", tags=["auth"])
    application.include_router(pessoas.router, prefix="/pessoas", tags=["pessoas"])
    application.include_router(ping.router, prefix="/ping", tags=["ping"])

    return application


app = create_application()


@app.on_event("startup")
async def startup_event():
    log.info("Starting up...")


@app.on_event("shutdown")
async def shutdown_event():
    log.info("Shutting down...")



from tortoise import fields, models


class User(models.Model):
    id = fields.IntField(pk=True)
    active = fields.BooleanField(default=True)
    client_id = fields.CharField(max_length=50, null=True)
    email = fields.CharField(max_length=200, null=True)
    scope = fields.TextField(has_db_field=False)
    username = fields.CharField(max_length=50, null=False)
    password_hash = fields.CharField(max_length=300, null=False)
    permissions: fields.ReverseRelation["UserPermission"]

    def __str__(self) -> str:
        return self.username

    class Meta:
        table = "users"


class Client(models.Model):
    id = fields.IntField(pk=True)
    active = fields.BooleanField()
    activity_sectors_id = fields.IntField(null=False)
    full_name = fields.CharField(max_length=300, null=True)
    image_url = fields.CharField(max_length=300, null=True)
    logo_url = fields.CharField(max_length=300, null=True)
    multiple_domains = fields.BooleanField(null=True, default=False)
    name = fields.CharField(max_length=50)
    valid_domains = fields.CharField(max_length=500, null=True)

    class Meta:
        table = "clients"


class Permission(models.Model):
    id = fields.IntField(pk=True)
    name = fields.CharField(max_length=50)
    description = fields.CharField(max_length=50)
    created_at = fields.DatetimeField(null=True)
    updated_at = fields.DatetimeField(null=True)

    class Meta:
        table = "permissions"


class UserPermission(models.Model):
    id = fields.IntField(pk=True)
    client_id = fields.CharField(max_length=50)
    entities = fields.CharField(max_length=3000)

    permission: fields.ForeignKeyRelation[Permission] = fields.ForeignKeyField(
        "models.Permission", related_name="permission"
    )
    user: fields.ForeignKeyRelation[User] = fields.ForeignKeyField(
        "models.User", related_name="permissions"
    )

    class Meta:
        table = "users_permissions"



import os
from typing import List

from fastapi import (APIRouter, Depends, File, HTTPException, Security,
                     UploadFile)
from openpyxl import load_workbook
from pydantic.error_wrappers import ValidationError

from src.epona.auth import schemas as auth
from src.epona.auth.services import get_current_active_user

from . import schemas, services

router = APIRouter()


@router.post("", response_model=schemas.PessoaResponseSchema, status_code=201)
async def create(
    payload: schemas.PessoaPayloadSchema,
    user: auth.UserSchema = Security(
        get_current_active_user, scopes=["pessoas:create"]
    ),
) -> schemas.PessoaResponseSchema:
    pessoa_id = await services.post(payload, user)

    response_object = payload.dict()
    response_object["id"] = pessoa_id

    return schemas.PessoaResponseSchema(**response_object)


@router.put("", response_model=schemas.PessoaResponseSchema, status_code=201)
async def update(
    payload: schemas.PessoaPayloadSchema,
    user: auth.UserSchema = Security(
        get_current_active_user, scopes=["pessoas:update"]
    ),
) -> schemas.PessoaResponseSchema:
    pessoa_id = await services.update(payload, user)
    if not pessoa_id:
        raise HTTPException(status_code=401, detail="Não autorizado")
    response = payload.dict()
    response["id"] = pessoa_id
    return schemas.PessoaResponseSchema(**response)


@router.get("/{pk}", response_model=schemas.PessoaSchema)
async def read(
    pk: int,
    user: auth.UserSchema = Security(get_current_active_user, scopes=["pessoas:read"]),
) -> schemas.Pessoa:
    if not user:
        raise HTTPException(status_code=404, detail="Usuário não autenticado")
    pes = await services.get(pk, user)
    if not pes:
        raise HTTPException(status_code=404, detail="Pessoa não encontrada")

    return pes


@router.get("/cpf_cnpj/{cpf_cnpj}", response_model=schemas.PessoaSchema)
async def cpf_cnpj(
    cpf_cnpj: str,
    user: auth.UserSchema = Security(get_current_active_user, scopes=["pessoas:read"]),
) -> schemas.PessoaSchema:
    pes = await services.by_cpf_cnpj(cpf_cnpj, user)
    if not pes:
        raise HTTPException(status_code=404, detail="Pessoa não encontrada")
    return dict(pes)


@router.get("", response_model=List[schemas.PessoaSchema])
async def read_all(
    user: auth.UserSchema = Security(get_current_active_user, scopes=["pessoas:read"])
) -> List[schemas.PessoaSchema]:
    if not user:
        raise HTTPException(status_code=404, detail="Usuário não autenticado")
    return await services.get_all(user)


@router.delete("/{pk}", status_code=200)
async def delete(
    pk: int,
    user: auth.UserSchema = Security(
        get_current_active_user, scopes=["pessoas:delete"]
    ),
) -> dict:
    result = await services.delete(pk, user)
    if result:
        return {"detail": "Pessoa excluída"}
    raise HTTPException(status_code=400, detail="Pessoa não encontrada")


@router.get("/principal/{pk}", status_code=200)
async def find_principal(
    pk: int,
    user: auth.UserSchema = Security(get_current_active_user, scopes=["pessoas:read"]),
) -> schemas.PessoaResponseSchema:
    principal = await services.find_principal(pk, user)
    if not principal:
        raise HTTPException(status_code=404, detail="Pessoa principal não encontrada")
    return principal


@router.post(
    "/xlsx", response_model=List[schemas.PessoaResponseSchema], status_code=201
)
async def process_file(
    upload: UploadFile = File(...),
    user: auth.UserSchema = Security(
        get_current_active_user, scopes=["pessoas:create"]
    ),
) -> List[schemas.PessoaResponseSchema]:
    if not upload:
        raise HTTPException(detail="Nenhum arquivo enviado", status_code=404)
    with open(f"{user.client_id}_{user.username}_tempfile.xlsx", "wb") as xlsx:
        contents = upload.file.read()
        xlsx.write(contents)
        wb = load_workbook(xlsx.name, read_only=True)
        try:
            result = await services.save_xlsx(wb, user)
        except (ValueError, ValidationError, Exception) as err:
            raise HTTPException(detail=str(err), status_code=422)
        finally:
            os.remove(xlsx.name)
        if result is None:
            raise HTTPException(
                detail="Falha na leitura dos dados. Comunique o administrador",
                status_code=404,
            )
        return result


@router.post(
    "/contato", response_model=schemas.InfoContatoResponseSchema, status_code=201
)
async def save_contato(
    payload: schemas.InfoContatoPayloadSchema,
    user: auth.UserSchema = Security(
        get_current_active_user, scopes=["pessoas:create"]
    ),
) -> schemas.InfoContatoResponseSchema:
    result = await services.save_contato(0, payload, user)
    if not result:
        raise HTTPException(status_code=400, detail="Falhar ao salvar")
    return result


@router.delete("/contato/{pk}", response_model=bool)
async def delete_contato(
    pk: int,
    user: auth.UserSchema = Security(
        get_current_active_user, scopes=["pessoas:delete"]
    ),
) -> bool:
    result = await services.delete_contato(pk, user)
    if not result:
        raise HTTPException(status_code=404, detail="InfoContato não encontrada")
    return result


@router.put("/contato/{pk}", response_model=schemas.InfoContatoResponseSchema)
async def update_contato(
    pk: int,
    payload: schemas.InfoContatoPayloadSchema,
    user: auth.UserSchema = Security(
        get_current_active_user, scopes=["pessoas:delete"]
    ),
) -> schemas.InfoContatoResponseSchema:
    result = await services.save_contato(pk, payload, user)
    if not result:
        raise HTTPException(status_code=404, detail="InfoContato não encontrada")
    return result


@router.get("/contato/{pk}", response_model=schemas.InfoContatoResponseSchema)
async def get_contato(
    pk: int, user: auth.UserSchema = Depends(get_current_active_user)
) -> schemas.InfoContatoResponseSchema:
    result = await services.get_contato(pk, user=user)
    if not result:
        raise HTTPException(status_code=404, detail="InfoContato não encontrada")
    return result


@router.get(
    "/contato/all/{pessoa_id}", response_model=List[schemas.InfoContatoResponseSchema]
)
async def get_contato_all(
    pessoa_id: int,
    user: auth.UserSchema = Security(get_current_active_user, scopes=["pessoas:read"]),
) -> List[schemas.InfoContatoResponseSchema]:
    result = await services.get_contato_all(pessoa_id, user)
    if not result:
        raise HTTPException(
            status_code=404, detail="Nenhuma informação de contato encontrada"
        )
    return result



from datetime import timedelta
from typing import List, Optional

from fastapi import APIRouter, Depends, HTTPException, Security, status
from fastapi.security import OAuth2PasswordRequestForm

from . import schemas, services

ACCESS_TOKEN_EXPIRE_MINUTES = 60 * 12
router = APIRouter()


@router.post("/login", response_model=schemas.Token)
async def login(form_data: OAuth2PasswordRequestForm = Depends()) -> dict:
    user = await services.authenticate_user(
        form_data.username, form_data.password, form_data.client_id
    )

    if not user:
        raise HTTPException(
            status_code=400,
            detail="Usuário ou senha incorreto",
            headers={"WWW-Authenticate": "Bearer"},
        )
    access_token_expires = timedelta(minutes=ACCESS_TOKEN_EXPIRE_MINUTES)
    access_token = services.create_access_token(
        data={"sub": f"{form_data.client_id}:{user.username}", "scopes": user.scope},
        expires_delta=access_token_expires,
    )
    return {"access_token": access_token, "token_type": "bearer"}


@router.post("/register", response_model=schemas.Token)
async def register(
    form_data: schemas.UserForm = Depends(schemas.UserForm.as_form),
) -> dict:
    http_exception = HTTPException(
        status_code=400,
        detail=(
            "Nao foi possível criar o usuário. Verifique se o e-mail e o CPF são "
            "únicos na empresa, se o nome escolhido para empresa está disponível "
            "ou e se o domínio do e-mail pode ser vinculado à empresa."
        ),
        headers={"WWW-Authenticate": "Bearer"},
    )
    if not form_data.password or not form_data.email:
        raise http_exception
    user_dict = form_data.dict()
    hashed_password = services.get_password_hash(form_data.password)
    user_dict["password_hash"] = hashed_password
    user_dict.pop("password", None)
    user_dict["id"] = 1

    user = schemas.UserSchema(**user_dict)
    user_wp = await services.save(user)
    if not user_wp:
        raise http_exception
    access_token_expires = timedelta(minutes=ACCESS_TOKEN_EXPIRE_MINUTES)
    access_token = services.create_access_token(
        data={"sub": f"{user.client_id}:{user_wp.username}", "scopes": user_wp.scope},
        expires_delta=access_token_expires,
    )
    return {"access_token": access_token, "token_type": "bearer"}


@router.get("/users/me")
async def read_users_me(
    current_user: schemas.UserSchema = Depends(services.get_current_active_user),
) -> schemas.UserSchema:
    return current_user


@router.get("/users/{username}", response_model=schemas.UserPermissionResponseSchema)
async def get_user_detail(
    username: str,
    user: schemas.UserSchema = Security(
        services.get_current_active_user, scopes=["admin"]
    ),
) -> Optional[schemas.UserPermissionResponseSchema]:
    try:
        user = await services.get_user_detail(username, user)
        return user
    except (HTTPException, Exception):
        raise HTTPException(
            status_code=status.HTTP_404_NOT_FOUND,
            detail=f"Usuário {username} inexistente",
        )


@router.get("/users", response_model=List[schemas.UserSchema])
async def get_users(
    user: schemas.UserSchema = Security(
        services.get_current_active_user, scopes=["admin"]
    )
) -> Optional[List[schemas.UserSchema]]:
    try:
        return await services.get_users(user)
    except HTTPException:
        raise HTTPException(
            status_code=status.HTTP_404_NOT_FOUND, detail="Nenhum usuário encontrado"
        )


@router.delete("/users/user-permission", response_model=int)
async def delete_user_permission(
    payload: schemas.UserPermissionPayloadSchema,
    _: schemas.UserSchema = Security(
        services.get_current_active_user, scopes=["admin"]
    ),
) -> int:
    return await services.delete_user_permission(payload)


@router.delete("/users/{username}", response_model=bool)
async def delete_user(
    username: str,
    user: schemas.UserSchema = Security(
        services.get_current_active_user, scopes=["admin", "users:delete"]
    ),
) -> bool:
    try:
        return await services.delete(username, user)
    except HTTPException:
        raise HTTPException(
            status_code=status.HTTP_404_NOT_FOUND,
            detail=f"Usuário {username} não encontrado",
        )


@router.put("/users", response_model=schemas.UserForm, status_code=200)
async def update_user(
    payload: schemas.UserForm = Depends(schemas.UserForm.as_form),
    user: schemas.UserSchema = Security(
        services.get_current_active_user, scopes=["admin", "users:update"]
    ),
) -> schemas.UserSchema:
    try:
        return await services.update(payload, user)
    except HTTPException:
        raise HTTPException(
            status.HTTP_404_NOT_FOUND, f"Usuário {payload.username} não encontrado"
        )


@router.get("/users/get-current/{token}")
async def get_current_user(token: str):
    security_scopes = Security()
    return await services.get_current_user(security_scopes, token)


@router.post("/users/permission", response_model=int, status_code=201)
async def user_add_permission(
    payload: schemas.UserPermissionPayloadSchema,
    user: schemas.UserSchema = Security(
        services.get_current_active_user, scopes=["admin"]
    ),
) -> int:
    return await services.user_add_permission(payload, user)


@router.post("/permission", response_model=bool, status_code=201)
async def add_permission(
    payload: schemas.PermissionSchema,
    _: schemas.UserSchema = Security(
        services.get_current_active_user, scopes=["super-admin"]
    ),
):
    try:
        return await services.add_permission(payload)
    except HTTPException:
        raise HTTPException(
            status_code=status.HTTP_204_NO_CONTENT,
            detail="Não foi possível incluir a permissão",
        )
    except Exception:
        raise HTTPException(
            status_code=status.HTTP_206_PARTIAL_CONTENT,
            detail="Não foi possível atribuir a permissão aos usuários admin",
        )


@router.get("/permissions", response_model=List[schemas.PermissionSchema])
async def get_permissions(
    _: schemas.UserSchema = Security(services.get_current_active_user, scopes=["admin"])
) -> Optional[List[schemas.PermissionSchema]]:
    return await services.get_permissions()


@router.post("/password-recover", response_model=Optional[schemas.EmailResponse])
async def password_recover(
    payload: schemas.PasswordRecoverPayload,
) -> Optional[schemas.EmailResponse]:
    return await services.password_recover(payload)


@router.post("/recovering", response_model=Optional[schemas.EmailResponse])
async def recovering(
    form_data: schemas.ChangePassword = Depends(schemas.ChangePassword.as_form),
) -> Optional[schemas.EmailResponse]:
    result = await services.recovering(form_data)
    if not result:
        raise HTTPException(401, "Token inválido")
    return result


@router.post("/password-change", response_model=Optional[schemas.EmailResponse])
async def password_change(
    form_data: schemas.ChangePassword = Depends(schemas.ChangePassword.as_form),
    user: schemas.UserSchema = Depends(services.get_current_active_user),
) -> Optional[schemas.EmailResponse]:
    result = await services.change_password(form_data, user)
    return result




import io
import os

from setuptools import setup

# Package metadata
NAME = "epona-api-core"
DESCRIPTION = "Epona API-Core"
EMAIL = "marcos@eponaconsultoria.com.br"
AUTHOR = "Antonio Marcos"
REQUIRES_PYTHON = ">=3.9.0"
VERSION = "0.0.15"

# Required packages
REQUIRED = [
    "fastapi==0.95.2",
    "asyncpg==0.27.0",
    "boto3==1.28.18",
    "jinja2==3.1.2",
    "pydantic==1.10.7",
    "python-multipart==0.0.6",
    "python-jose==3.3.0",
    "tortoise-orm==0.19.3",
    "passlib[bcrypt]",
]

current_dir = os.path.abspath(os.path.dirname(__file__))

# Import the README and use it as long-description
try:
    with io.open(os.path.join(current_dir, "README.md"), encoding="utf-8") as f:
        long_description = f"\n{f.read()}"
except FileNotFoundError:
    long_description = DESCRIPTION

# Load the package's __version__.py module as a dictionary
about = {}
if not VERSION:
    project_slug = NAME.lower().replace("-", "_").replace(" ", "_")
    with open(os.path.join(current_dir, project_slug, "__version__.py")) as f:
        exec(f.read(), about)
else:
    about['__version__'] = VERSION

setup(
    name=NAME,
    version=about['__version__'],
    description=DESCRIPTION,
    long_description=long_description,
    long_description_content_type="text/markdown",
    author=AUTHOR,
    author_email=EMAIL,
    python_requires=REQUIRES_PYTHON,
    install_requires=REQUIRED,
    include_package_data=True,
    classifiers=[
        "Programming Language :: Python :: 3.9",
    ],
    setup_requires=['wheel']
)
